;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of Area51Raid.
;;
;;    Area51Raid is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Area51Raid is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Area51Raid.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2019 edition.
;;
;;    Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;    Aguirre Ivorra, Antonio              antoni.aguirre@gmail.com
;;    López Gutiérrez, Juan Carlos         juancarlos.logut@gmail.com
;;    Pérez Ponsoda, Jorge Amil            j.a.perez.ponsoda@gmail.com
;;
;;########################### END GPL LICENSE BLOCK #############################


;;
;;  ENTITY MANAGER
;;
.include "cpctelera.h.s"
.include "man/entity.h.s"
.include "man/entity_ai.h.s"
.include "man/render.h.s"
.include "man/patrol.h.s"
.include "man/physics.h.s"
.include "cmp/array.h.s"
.include "cmp/entity.h.s"
.include "assets/assets.h.s"
.include "man/projectile.h.s"
;;.include "man/components.h.s"
.include "man/collision.h.s"

.module entity_manager

en_ar::
DefineComponentArrayStructure _entity, max_entities, DefineCmp_Entity_default

;;  INPUT:
;;      -
;;  OUTPUT:
;;      HL: Pointer to the entity array
entity_man_getArray::
    ld hl, #_entity_array
    ret

;;  INPUT:
;;      -
;;  OUTPUT:
;;      HL: Pointer to the entity array
;;  DESTROY:
;;      DE, A
entity_man_init::
;; ponemos a cero el numero de entidades
    xor a                           ;; [1]
    ld (_entity_num), a             ;; [4]
;; ponemos el puntero a la ultima posicion
    ld hl, #_entity_array           ;; [3]
    ld (_entity_pend), hl           ;; [5]
;;ponemos el puntro en de
    ex de, hl
;; calculamos el offset el acceso a la variable ancho
    ld hl, #e_w
    add hl, de
;; ponemos el valor de la ultima entidad como invalid
    ld (hl), #e_w_invalidEntity
;; recuperamos el puntero
    ex de, hl
    ret                             ;; [3]

;;  INPUT
;;      HL: Pointer to entity initializer bytes
;;  DESTROY:
;;      HL, DE, BC, A
entity_man_create::
    push hl                          ;; [4]
    call entity_man_new              ;; [x]

    ld__ixh_d         ;; [2]
    ld__ixl_e         ;; [2]

    pop hl            ;; [3]
    ldir              ;; [x]

;; ejemplo para añadir entidades a un array d punteros
    ld a, e_cmp_type(ix)
    and #e_cmp_AI
;; sustituir ==============
    call nz, entity_ai_man_add
;;    jr z, _noAI
;;_AI:
;;    ld a, #e_cmpID_AI
;;    call components_man_add
;;_noAI:
;;    ld a, e_cmp_type(ix)
;;    and #e_cmp_Physics
;;    jr z, _noPhys
;;_Phys:
;;    ld a, #e_cmpID_Physics
;;    call components_man_add
;;_noPhys:
;;    ret

    ld a, e_cmp_type(ix)
    and #e_cmp_Collision
    call nz, collision_man_add

    ld a, e_cmp_type(ix)
    and #e_cmp_Physics
    call nz, physics_man_add

    ld a, e_cmp_type(ix)
    and #e_cmp_Render
    call nz, entity_render_man_add

    ld a, e_cmp_type(ix)
    and #e_cmp_Patrol
    call nz, patrol_man_add

    ld  a,  e_cmp_type(ix)
    and #e_cmp_Proyectil
    call nz, projectile_man_add  
    
    ret               ;; [3]
;;=============

;;  INPUT:
;;      -
;;  OUTPUT:  
;;      DE: Pointer to the new entity
;;      BC: Entity size
;;  DESTROY:
;;      HL
entity_man_new::
    ld hl, #_entity_num             ;; [3]
    inc (hl)                        ;; [2]

    ld hl, (_entity_pend)           ;; [5]
    ld d, h                         ;; [1]
    ld e, l                         ;; [1]
    ld bc, #sizeof_e                ;; [3]
    add hl, bc                      ;; [3]
    ld (_entity_pend), hl           ;; [5]

    push de

    ld de, #e_w
    add hl, de
    ld (hl), #e_w_invalidEntity

    pop de

    ret                             ;; [3]
