;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of Area51Raid.
;;
;;    Area51Raid is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Area51Raid is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Area51Raid.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2019 edition.
;;
;;    Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;    Aguirre Ivorra, Antonio              antoni.aguirre@gmail.com
;;    López Gutiérrez, Juan Carlos         juancarlos.logut@gmail.com
;;    Pérez Ponsoda, Jorge Amil            j.a.perez.ponsoda@gmail.com
;;
;;########################### END GPL LICENSE BLOCK #############################


;;
;; GAME MANAGER
;;
;; CPCTELERA UTILS
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
;; GAME
.include "man/game.h.s"
;; MENU
.include "man/menu.h.s"
;; LEVEL
.include "man/level.h.s"
;; ENTITY MANAGER
.include "man/entity.h.s"
;; AI
.include "man/entity_ai.h.s"
.include "sys/entity_ai.h.s"
;; RENDER
.include "man/render.h.s"
.include "sys/render.h.s"
;; PATROL
.include "man/patrol.h.s"
.include "sys/patrol.h.s"
;; PROJECTILE
.include "man/projectile.h.s"
.include "sys/projectile.h.s"
;; INPUT
.include "sys/input.h.s"
;; GAME STATUS
.include "sys/game_status.h.s"
;; PHYSICAS
.include "man/physics.h.s"
.include "sys/physics.h.s"
;; COLISION
.include "man/collision.h.s"
.include "sys/collision.h.s"
; PLayer
.include "cmp/entity.h.s"
.include "assets/assets.h.s"
;; Animations
;; .include "sys/animations.h.s"

.module game_manager

player_default_hp:: .db 10
player_hp::         .db 10

;; Se inician todas las variables necesarias del juego
game_man_init::
    ;; Inicializacion managers
    ;call cpct_akp_musicPlay_asm
    call collision_man_init
    call entity_ai_man_init
    call entity_render_man_init
    call patrol_man_init
    call physics_man_init
    call projectile_man_init
    ;; Inicializacion manager entidades
    call entity_man_init
    ;; Inicializacion systemas
    call input_sys_init
    call game_status_sys_init
    ;; Inicializacion systemas con manager
    call entity_ai_sys_init
    call physics_sys_init
    call render_sys_init
    call collision_sys_init
    call projectile_sys_init
    call patrol_sys_init
;;    call animation_sys_init

    ;; Inicializacion de las variables de game
    ld  hl, #next_level
    ld  (hl), #invalid_ptr
    ld  hl, #start_menu_man
    ld  (_next_menu), hl

    ret

start_game::
    ;; Si el puntero al siguiente nivel es invalido se llama al manejador de menu
    ld   a, (next_level)
    cp  #invalid_ptr

    jr  nz, load_level

    ;; Llamada al manejador de menu
_next_menu = .+1
    call #0x0000

    ;; Al salir del manejador de menu siempre se va a lanzar el primer nivel
    ld  hl, #init_start_level
    ld  (next_level), hl

    call reset_player_status

    jr  start_game

load_level:
    ;;Manejador de niveles
    ld  hl, (next_level)
    call run_level

    jr  start_game

;;  Se selecciona el siguiente nivel que se va a mostrar
;;  INPUT:
;;      HL -> pointer to the next level init
;;  DESTROY:
;;      HL
set_next_level_init::
    ld  (next_level), hl

    ret


;;  Se asigna el menu que se va a mostrar en la siguiente iteracion del bucle del juego
;;  INPUT:
;;      HL -> pointer to the next menu
;;  DESTROY:
;;      HL
set_next_menu_ptr::
    ld  (_next_menu), hl

    ld  hl, #next_level
    ld  (hl), #invalid_ptr

    ret


;;  DESTROY:
;;      HL, DE, BC;?
save_player_status::
    call entity_man_getArray
    ld  de, #e_hp
    add hl, de
    ld   a, (hl)
    ld  hl, #player_hp
    ld  (hl), a

    ret


;;  DESTROY:
;;      HL, DE, BC, A;?
load_player_status::
    call entity_man_getArray
    ld  de, #e_hp
    add hl, de
    ld   a, (player_hp)
    ld  (hl), a

    ret


;;  DESTROY:
;;      HL, A;?
reset_player_status::
    ld  hl, #player_hp
    ld   a, (player_default_hp)
    ld  (hl), a

    ret