;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of Area51Raid.
;;
;;    Area51Raid is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Area51Raid is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Area51Raid.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2019 edition.
;;
;;    Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;    Aguirre Ivorra, Antonio              antoni.aguirre@gmail.com
;;    López Gutiérrez, Juan Carlos         juancarlos.logut@gmail.com
;;    Pérez Ponsoda, Jorge Amil            j.a.perez.ponsoda@gmail.com
;;
;;########################### END GPL LICENSE BLOCK #############################


;;
;;  LEVEL MANAGER
;;
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
;; COMPONENTES
.include "cmp/entity.h.s"
.include "assets/assets.h.s"
;; GAME
.include "man/game.h.s"
;; MENU
.include "man/menu.h.s"
;; entity MANAGER
.include "man/entity.h.s"
;; Game status
.include "sys/game_status.h.s"
;; Render
.include "sys/render.h.s"
.include "man/render.h.s"
;; IA
.include "man/entity_ai.h.s"
.include "sys/entity_ai.h.s"
;; PATROL
.include "man/patrol.h.s"
.include "sys/patrol.h.s"
;; COLLISION
.include "man/collision.h.s"
.include "sys/collision.h.s"
;; PHYSICS
.include "man/physics.h.s"
.include "sys/physics.h.s"
;; INPUT
.include "sys/input.h.s"
;; PROY
.include "sys/projectile.h.s"
.include "man/projectile.h.s"
;; Animations
;; .include "sys/animations.h.s"

;;
;;  Se ejecuta el manejador de un nivel y devuelve el control cuado este termina
;;  INPUT:
;;      HL -> pointer to the init_menu
run_level::
    ld  (_next_level), hl
    
_next_level = .+1   
    call 0x0000

    ;; Se indica qe no tiene que continuar al siguiente nivel
    ld   a, #0x00
    ld  (_go_next_level), a

    ;; Cargamos el estado del jugador
    call load_player_status

    ;; Se pinta el fondo        ;# Pintar el fondo en ambos buffer
    call draw_background

    call patrol_sys_update

    ;; Se pinta una primera vez las entidades para aplicar XOR de manera par
    call init_render_sys_update

unfinished_level:
    ;; Bucle del nivel
    call update_level

    call cpct_waitVSYNC_asm
    call switchBuffers

    call render_sys_update

    ;; Comprobacion de avance o finalizacion de la partida
    call check_game_status

    jr  nz, unfinished_level

finished_level:
    ;; Se guarda el estado del jugador
    call save_player_status

    ;; Se borra el array de entidades
    call collision_man_init
    ;; Se borran los arrays de punteros
    call patrol_man_init
    call physics_man_init
    call entity_ai_man_init
    call entity_render_man_init
    call projectile_man_init
;;    call animation_sys_init ???
    ;; Se borra el array de entidades
    call entity_man_init
    
    ;; Borrar mapa
    call clear_screen

    ret


;;===================================================================================================

;;
;; Comprobacion del estado de la partida
;;
check_game_status:
    call check_player_status

    jr  nz, alive

    ;; Se modifica el menu que se va a mostrar
    ld  hl, #gamo_menu_man
    call set_next_menu_ptr
    ret

alive:

    ;; Se comprueba si se tiene que pasar de nivel
_go_next_level = .+1
    ld   a, #0x00
    cp  #0x01

    jr  z, not_next_level

    ret

not_next_level:

    ret


;;===================================================================================================

go_to_next_level::
    ld  hl, (_go_next_level)
    ld   a, #0x01
    ld  (_go_next_level), a

    ret

;;
;;  Actualizacion de los sistemas del nivel
;;
update_level::
    call input_sys_update
    call entity_ai_sys_update
    call projectile_sys_update
    call physics_sys_update
;    call animation_sys_update
    call collision_sys_update

    ret

;;  ################################
;;  # Inicializacion de cada nivel #
;;  ################################

;; declaracion entidades de prueba
;;player:   DefineCmp_Entity    0, 100, 1, 1, 6, 15,  _sp_naruto,     e_ai_st_noAI, 10, e_cmp_jugador
;;playerDe: DefineCmp_Entity   70, 100, 1, 1, 6, 15,  _sp_naruto,     e_ai_st_noAI, 10, e_cmp_jugador
;;playerAl: DefineCmp_Entity    8, 100, 1, 1, 6, 15,  _sp_naruto,     e_ai_st_noAI, 10, e_cmp_jugador
;;alien:    DefineCmp_Entity   70, 100, 0, 0, 7, 15,   _sp_alien,     e_ai_st_seek, 10, e_cmp_et
;;puertaIz: DefineCmp_Entity    0,   0, 0, 0, 2, 450,      0x0000, e_ai_st_stand_by, 10, e_cmp_door
;;puertaDe: DefineCmp_Entity   79,   0, 0, 0, 2, 450,      0x0000, e_ai_st_stand_by, 10, e_cmp_door
;;
;;disparo1:   DefineCmp_Entity 64,20, -1, 1, 3, 5, _sp_projectile, e_ai_st_noAI, 100, e_cmp_bullet
;;disparo2:   DefineCmp_Entity 64,120, -1, -1, 3, 5, _sp_projectile, e_ai_st_noAI, 100, e_cmp_bullet
;;soldado_torre1: DefineCmp_Entity 72,20, 0, 0, 7, 16, _sp_soldier, e_ai_st_stand_by, 100, e_cmp_ai_entity
;;soldado_torre2: DefineCmp_Entity 72,120, 0, 0, 7, 16, _sp_soldier, e_ai_st_stand_by, 100, e_cmp_ai_entity
;;
;;ent1:   DefineCmp_Entity 40,  20, 0, 0, 7, 16, _sp_soldier, e_ai_st_patrol, 10, e_cmp_ai_entity_patrol
;;ent2:   DefineCmp_Entity 40, 180, 0, 0, 7, 16, _sp_soldier, e_ai_st_patrol, 10, e_cmp_ai_entity_patrol
;;;;ent3:   DefineCmp_Entity 20, 120, 0, 0, 7, 16, _sp_soldier, e_ai_st_patrol, 10, e_cmp_ai_entity_patrol
;;player:         DefineCmp_Entity    0,  100,  1,  1,  6,  15,       _sp_naruto,      e_ai_st_noAI,   10,          e_cmp_jugador,   e_colide_st_player
;;playerDe:       DefineCmp_Entity   70,  100,  1,  1,  6,  15,       _sp_naruto,      e_ai_st_noAI,   10,          e_cmp_jugador,   e_colide_st_player
;;playerAl:       DefineCmp_Entity    8,  100,  1,  1,  6,  15,       _sp_naruto,      e_ai_st_noAI,   10,          e_cmp_jugador,   e_colide_st_player
;;
;;alien:          DefineCmp_Entity   70,  100,  0,  0,  7,  15,        _sp_alien,      e_ai_st_seek,   10,               e_cmp_ai,    e_colide_st_alien
;;
;;puertaIz:       DefineCmp_Entity    0,    0,  0,  0,  2, 200,           0x0000,  e_ai_st_stand_by,   10,        e_cmp_Collision,     e_colide_st_door
;;puertaDe:       DefineCmp_Entity   79,    0,  0,  0,  2, 200,           0x0000,  e_ai_st_stand_by,   10,        e_cmp_Collision,     e_colide_st_door
;;
;;disparo1:       DefineCmp_Entity   64,   20, -1,  0,  3,   5,   _sp_projectile,      e_ai_st_noAI,  100,           e_cmp_bullet,   e_colide_st_bullet
;;disparo2:       DefineCmp_Entity   64,  120, -1,  0,  3,   5,   _sp_projectile,      e_ai_st_noAI,  100,           e_cmp_bullet,   e_colide_st_bullet
;;soldado_torre1: DefineCmp_Entity   72,   20,  0,  0,  7,  16,      _sp_soldier,  e_ai_st_stand_by,  100,               e_cmp_ai,       e_colide_st_ia
;;soldado_torre2: DefineCmp_Entity   72,  120,  0,  0,  7,  16,      _sp_soldier,  e_ai_st_stand_by,  100,               e_cmp_ai,       e_colide_st_ia
;;
;;ent1:           DefineCmp_Entity   40,  20,   0,  0,  7,  16,      _sp_soldier,    e_ai_st_patrol,   10,        e_cmp_ai_patrol,       e_colide_st_ia
;;ent2:           DefineCmp_Entity   40, 180,   0,  0,  7,  16,      _sp_soldier,    e_ai_st_patrol,   10,        e_cmp_ai_patrol,       e_colide_st_ia
;;ent3:           DefineCmp_Entity   20, 120,   0,  0,  7,  16,      _sp_soldier,    e_ai_st_patrol,   10,        e_cmp_ai_patrol,       e_colide_st_ia

;muro_doble: DefineCmp_Entity 60, 50, 0, 0, 16, 16, _sp_muro_doble, e_ai_st_stand_by, 10, e_cmp_default
;;puertaDe: DefineCmp_Entity   80,  100, 20, 10, 2, 450,      0x0000, e_ai_st_stand_by, 10, e_cmp_door

;;                               #######################
;;                               #  MAPAS DE INCURSION #
;;                               #######################



;;  ####################################################
;;                          nivel 0
;;  ####################################################
puerta_1:   DefineCmp_Entity    79, 85, 0, 0, 2, 30,      0x0000,   e_ai_st_stand_by,   0,  e_cmp_Collision,    e_colide_st_door
player_1:   DefineCmp_Entity    10, 85, 1, 1, 6, 15,  _sp_naruto,   e_ai_st_noAI,       10, e_cmp_jugador,      e_colide_st_player
muro_1_0:   DefineCmp_Entity    0, 0, 0, 0, 160, 70,  0x0000,       e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
muro_1_1:   DefineCmp_Entity    0, 120, 0, 0, 160, 110,  0x0000,    e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia

init_start_level::
    ld  hl, #_level0_pack_end
    ld  de, #decompress_buffer_end_0
    call cpct_zx7b_decrunch_s_asm
    ld  bc, #0x1914 
    ld  de, #_level0_W
    ld  hl, #tileset_ptr    
    call cpct_etm_setDrawTilemap4x8_ag_asm

    ld  hl, #map_1
    call set_background
    
    ;; jugador
    ld    hl,      #player_1
    call  entity_man_create

    ;; puerta
    ld    hl,      #puerta_1
    call  entity_man_create
    
    ;; muros
    ld  hl, #muro_1_0
    call    entity_man_create
    ld  hl, #muro_1_1
    call    entity_man_create

    ;; next level
    ld   hl, #init_level_2
    call set_next_level_init
    
    ret

;;  ####################################################
;;                          nivel 2
;;  ####################################################
muro_2_0:       DefineCmp_Entity    40, 48, 0, 0, 23, 88,  0x0000,      e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
muro_2_1:       DefineCmp_Entity    0, 0, 0, 0, 160, 24,  0x0000,       e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
muro_2_2:       DefineCmp_Entity    0, 165, 0, 0, 160, 30,  0x0000,     e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
patrol_2_0:     DefineCmp_Entity    65,  140, 0, 0, 7, 16, _sp_soldier, e_ai_st_patrol,     10, e_cmp_ai_patrol,    e_colide_st_ia

init_level_2::
    ;; tilemap
    ld  hl, #map_2
    call set_background

    ;; jugador
    ld    hl,      #player_1
    call  entity_man_create

    ;; puerta
    ld    hl,      #puerta_1
    call  entity_man_create
;;
    ;; muros
    ld  hl, #muro_2_0
    call    entity_man_create
    ld  hl, #muro_2_1
    call    entity_man_create
    ld  hl, #muro_2_2
    call    entity_man_create

    ;; patrol
    ld  hl, #patrol_2_0
    call    entity_man_create
    

    ;; next level
    ld   hl, #init_level_3
    call set_next_level_init 
    ret

;;  ####################################################
;;                          nivel 3
;;  ####################################################
muro_3_0:       DefineCmp_Entity    16, 48, 0, 0, 10, 87,       0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
patrol_3_0:     DefineCmp_Entity    30, 140, 0, 0, 7, 16,       _sp_soldier,    e_ai_st_patrol,     10, e_cmp_ai_patrol,    e_colide_st_ia


init_level_3::
    ld  hl, #map_3
    call set_background

    ;; jugador
    ld    hl,      #player_1
    call  entity_man_create
    
    ;; puerta
    ld    hl,      #puerta_1
    call  entity_man_create

    ;; muros
    ld  hl, #muro_2_0
    call    entity_man_create
    ld  hl, #muro_2_1
    call    entity_man_create
    ld  hl, #muro_2_2
    call    entity_man_create
 
    ld  hl, #muro_3_0
    call    entity_man_create

    ;; patrol
    ld  hl, #patrol_2_0
    call    entity_man_create
    ld  hl, #patrol_3_0
    call    entity_man_create


    ld   hl, #init_level_4
    call set_next_level_init
    ret

;;  ####################################################
;;                          nivel 4
;;  ####################################################
muro_4_0:   DefineCmp_Entity    0, 0, 0, 0, 63, 80,       0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall
muro_4_1:   DefineCmp_Entity    32, 80, 0, 0, 31, 87,     0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall
muro_4_2:   DefineCmp_Entity    0, 112, 0, 0, 19, 90,     0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall

pincho_4_1:       DefineCmp_Entity    29, 106, 0, 0, 5, 23,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_4_2:       DefineCmp_Entity    19, 163, 0, 0, 4, 35,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_4_3:       DefineCmp_Entity    37, 150, 0, 0, 20, 25,  0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_4_4:       DefineCmp_Entity    61, 153, 0, 0, 6, 8,    0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_4_5:       DefineCmp_Entity    73, 115, 0, 0, 6, 10,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_4_6:       DefineCmp_Entity    61, 53, 0, 0, 6, 10,    0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia

puerta_4:   DefineCmp_Entity    65,   20, 0, 0, 10, 15,   0x0000,   e_ai_st_stand_by,   0,  e_cmp_Collision,    e_colide_st_door

init_level_4::
    ld  hl, #map_4
    call set_background

    ;; jugador
    ld    hl,      #player_1
    call  entity_man_create 

    ;; muros
    ld  hl, #muro_4_0
    call    entity_man_create
    ld  hl, #muro_4_1
    call    entity_man_create
    ld  hl, #muro_4_2
    call    entity_man_create

    ;; pinchos
    ld  hl, #pincho_4_1
    call    entity_man_create
    ld  hl, #pincho_4_2
    call    entity_man_create
    ld  hl, #pincho_4_3
    call    entity_man_create
    ld  hl, #pincho_4_4
    call    entity_man_create
    ld  hl, #pincho_4_5
    call    entity_man_create
    ld  hl, #pincho_4_6
    call    entity_man_create

    ;; puerta 4
    ld  hl, #puerta_4
    call    entity_man_create

    ld   hl, #init_level_5
    call set_next_level_init    
    ret

;;  ####################################################
;;                          nivel 5
;;  ####################################################
player_5:   DefineCmp_Entity    10, 160, 1, 1, 6, 15, _sp_naruto,   e_ai_st_noAI,       10, e_cmp_jugador,      e_colide_st_player

muro_5_0:   DefineCmp_Entity    0, 0, 0, 0, 160, 40,      0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall
muro_5_1:   DefineCmp_Entity    20, 64, 0, 0, 160, 160,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall

pincho_5_1:       DefineCmp_Entity    0, 64, 0, 0, 7, 25,     0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_5_2:       DefineCmp_Entity    17, 104, 0, 0, 7, 25,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_5_3:       DefineCmp_Entity    0, 144, 0, 0, 7, 25,    0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia

disparo_5:   DefineCmp_Entity 10, 40, 0, 1, 3, 5,             _sp_projectile, e_ai_st_noAI,       100,e_cmp_bullet,       e_colide_st_bullet

puerta_5:   DefineCmp_Entity    70,   50, 0, 0, 10, 15,      0x0000,   e_ai_st_stand_by,   0,  e_cmp_Collision,    e_colide_st_door
init_level_5:
    ld  hl, #map_5
    call    set_background

    ;; jugador
    ld    hl,      #player_5
    call  entity_man_create 

    ;; muros
    ld  hl, #muro_5_0
    call    entity_man_create
    ld  hl, #muro_5_1
    call    entity_man_create

    ;; pinchos
    ld  hl, #pincho_5_1
    call    entity_man_create
    ld  hl, #pincho_5_2
    call    entity_man_create
    ld  hl, #pincho_5_3
    call    entity_man_create

    ;; proyectil
    ld  hl, #disparo_5
    call    entity_man_create

    ;; puerta 
    ld  hl, #puerta_5
    call    entity_man_create

    ld   hl, #init_level_6
    call set_next_level_init
    
    ret

;;  ####################################################
;;                          nivel 6
;;  ####################################################
muro_8_0:   DefineCmp_Entity    0, 0, 0, 0, 80, 80,       0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall
pincho_6_1:       DefineCmp_Entity    32, 80, 0, 0, 12, 8,     0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_6_2:       DefineCmp_Entity    16, 110, 0, 0, 7, 8,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
pincho_6_3:       DefineCmp_Entity    52, 110, 0, 0, 7, 8,    0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
muro_7_0:   DefineCmp_Entity    0, 120, 0, 0, 160, 110,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall

init_level_6:
    ld  hl, #_level1_pack_end
    ld  de, #decompress_buffer_end_1
    call cpct_zx7b_decrunch_s_asm

    ld  bc, #0x1914 ;; tiles en pantalla, 18x18
    ld  de, #_level1_W
    ld  hl, #tileset_ptr_2    ;; puntero a donde se ubica el tileset
    ;; destroy AF, DE
    call cpct_etm_setDrawTilemap4x8_ag_asm

    ld  hl, #map_2
    call    set_background

    ;; jugador
    ld  hl, #player_1
    call    entity_man_create

    ;; puerta
    ld  hl, #puerta_1
    call    entity_man_create

    ;; muro
    ld  hl, #muro_7_0
    call    entity_man_create
    ld  hl, #muro_8_0
    call    entity_man_create

    ;; pinchos
    ld  hl, #pincho_6_1
    call    entity_man_create
    ld  hl, #pincho_6_2
    call    entity_man_create
    ld  hl, #pincho_6_3
    call    entity_man_create

    ld   hl, #init_level_7
    call set_next_level_init
    
    ret

;;  ####################################################
;;                          nivel 7
;;  ####################################################
;;puerta_1:   DefineCmp_Entity    79, 85, 0, 0, 2, 30,      0x0000,   e_ai_st_stand_by,   0,  e_cmp_Collision,    e_colide_st_door
;;player_1:   DefineCmp_Entity    10, 85, 1, 1, 6, 15,  _sp_naruto,   e_ai_st_noAI,       10, e_cmp_jugador,      e_colide_st_player
;;alien:    DefineCmp_Entity      40, 20, 0, 0, 7, 15,         _sp_alien,      e_ai_st_seek,   10,               e_cmp_ai,    e_colide_st_alien
player_7:       DefineCmp_Entity    20, 80, 0, 0, 6, 15,    _sp_naruto,   e_ai_st_noAI,       10, e_cmp_jugador,      e_colide_st_player
alien:          DefineCmp_Entity   70,  100,  0,  0,  7,  15,        _sp_alien,      e_ai_st_seek,   10,               e_cmp_ai,    e_colide_st_alien
muro_7_1:   DefineCmp_Entity    0, 0, 0, 0, 31, 80,       0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall
muro_7_2:   DefineCmp_Entity    48, 0, 0, 0, 160, 200,     0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_wall
puerta_1_back:  DefineCmp_Entity    0, 85, 0, 0, 8, 15,      0x0000,   e_ai_st_stand_by,   0,  e_cmp_Collision,    e_colide_st_door

init_level_7:


    ld  hl, #map_1
    call set_background

    ;;jugador
    ld    hl,      #player_7
    call  entity_man_create

    ;; puerta
    ld    hl,      #puerta_1_back
    call  entity_man_create

    ;; alien
    ld  hl, #alien
    call    entity_man_create

    ;; muro
    ;;ld  hl, #muro_7_0
    ;;call    entity_man_create
    ;;ld  hl, #muro_7_1
    ;;call    entity_man_create
    ;;ld  hl, #muro_7_2
    ;;call    entity_man_create


    ld   hl, #init_level_8
    call set_next_level_init
    
    ret

;;  ####################################################
;;                          nivel 8
;;  ####################################################
player_1_back:  DefineCmp_Entity    64, 80, 0, 0, 6, 15,    _sp_naruto_flip,   e_ai_st_noAI,       10, e_cmp_jugador,      e_colide_st_player

init_level_8:
    ld  hl, #map_2
    call set_background

    ;;jugador
    ld    hl,      #player_1_back
    call  entity_man_create

    ;; puerta
    ld    hl,      #puerta_1_back
    call  entity_man_create

    ;; muro
    ld  hl, #muro_7_0
    call    entity_man_create
    ld  hl, #muro_8_0
    call    entity_man_create

    ;; pinchos
    ld  hl, #pincho_6_1
    call    entity_man_create
    ld  hl, #pincho_6_2
    call    entity_man_create
    ld  hl, #pincho_6_3
    call    entity_man_create

    ld   hl, #init_level_9
    call set_next_level_init
    ret

player_5_back:   DefineCmp_Entity    70,   50, 0, 0, 6, 15,    _sp_naruto_flip,   e_ai_st_noAI,       10, e_cmp_jugador,      e_colide_st_player
puerta_5_back:   DefineCmp_Entity    10, 160, 1, 1, 10, 15,      0x0000,   e_ai_st_stand_by,   0,  e_cmp_Collision,    e_colide_st_door
init_level_9:
    ld  hl, #_level0_pack_end
    ld  de, #decompress_buffer_end_0
    call cpct_zx7b_decrunch_s_asm
    ld  bc, #0x1914 
    ld  de, #_level0_W
    ld  hl, #tileset_ptr    
    call cpct_etm_setDrawTilemap4x8_ag_asm

    ld  hl, #map_5
    call set_background

    ld  hl, #player_5_back
    call    entity_man_create

    ld  hl, #puerta_5_back
    call    entity_man_create
    
    ;; muros
    ld  hl, #muro_5_0
    call    entity_man_create
    ld  hl, #muro_5_1
    call    entity_man_create

    ;; pinchos
    ld  hl, #pincho_5_1
    call    entity_man_create
    ld  hl, #pincho_5_2
    call    entity_man_create
    ld  hl, #pincho_5_3
    call    entity_man_create

    ;; proyectil
    ld  hl, #disparo_5
    call    entity_man_create

    ld   hl, #init_rescue_the_alien_level_1
    call set_next_level_init

    ret

;;                               ####################
;;                               #  MAPA DE RESCATE #
;;                               ####################
;;muro_9_0:   DefineCmp_Entity    0, 120, 0, 0, 50, 110,   0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
;;muro_9_1:   DefineCmp_Entity    0, 0, 0, 0, 50, 80,       0x0000,         e_ai_st_stand_by,   10, e_cmp_Collision,    e_colide_st_ia
player_4_back:  DefineCmp_Entity    65,   20, 0, 0, 6, 15,    _sp_naruto_flip,   e_ai_st_noAI,       10, e_cmp_jugador,      e_colide_st_player


init_rescue_the_alien_level_1:
    ld  hl, #map_4
    call set_background

    ;; jugador
    ld    hl,      #player_4_back
    call  entity_man_create 

    ;; muros
    ld  hl, #muro_4_0
    call    entity_man_create
    ld  hl, #muro_4_1
    call    entity_man_create
    ld  hl, #muro_4_2
    call    entity_man_create

    ;; pinchos
    ld  hl, #pincho_4_1
    call    entity_man_create
    ld  hl, #pincho_4_2
    call    entity_man_create
    ld  hl, #pincho_4_3
    call    entity_man_create
    ld  hl, #pincho_4_4
    call    entity_man_create
    ld  hl, #pincho_4_5
    call    entity_man_create
    ld  hl, #pincho_4_6
    call    entity_man_create

    ;; puerta 4
    ld  hl, #puerta_1_back
    call    entity_man_create

    ld   hl, #init_rescue_the_alien_level_2
    call set_next_level_init
    ret

init_rescue_the_alien_level_2:

    ld  hl, #map_3
    call set_background

    ;; jugador
    ld    hl,      #player_1_back
    call  entity_man_create
    
    ;; puerta
    ld    hl,      #puerta_1_back
    call  entity_man_create

    ;; muros
    ld  hl, #muro_2_0
    call    entity_man_create
    ld  hl, #muro_2_1
    call    entity_man_create
    ld  hl, #muro_2_2
    call    entity_man_create
 
    ld  hl, #muro_3_0
    call    entity_man_create

    ;; patrol
    ld  hl, #patrol_2_0
    call    entity_man_create
    ld  hl, #patrol_3_0
    call    entity_man_create

    ld   hl, #init_rescue_the_alien_level_3
    call set_next_level_init
    ret

init_rescue_the_alien_level_3:

    ld  hl, #map_2
    call set_background

    ;; tilemap
    ld  hl, #map_2
    call set_background

    ;; jugador
    ld    hl,      #player_1_back
    call  entity_man_create

    ;; puerta
    ld    hl,      #puerta_1_back
    call  entity_man_create
;;
    ;; muros
    ld  hl, #muro_2_0
    call    entity_man_create
    ld  hl, #muro_2_1
    call    entity_man_create
    ld  hl, #muro_2_2
    call    entity_man_create

    ;; patrol
    ld  hl, #patrol_2_0
    call    entity_man_create

    ld   hl, #init_rescue_the_alien_level_4
    call set_next_level_init
    ret

init_rescue_the_alien_level_4:

    ld  hl, #map_1
    call set_background

    ;; jugador
    ld    hl,      #player_1_back
    call  entity_man_create

    ;; puerta
    ld    hl,      #puerta_1_back
    call  entity_man_create

    ;; muros
    ld  hl, #muro_1_0
    call    entity_man_create
    ld  hl, #muro_1_1
    call    entity_man_create

    ld  hl, #win_menu_man
    call set_next_menu_ptr
    ret