;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of Area51Raid.
;;
;;    Area51Raid is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Area51Raid is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Area51Raid.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2019 edition.
;;
;;    Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;    Aguirre Ivorra, Antonio              antoni.aguirre@gmail.com
;;    López Gutiérrez, Juan Carlos         juancarlos.logut@gmail.com
;;    Pérez Ponsoda, Jorge Amil            j.a.perez.ponsoda@gmail.com
;;
;;########################### END GPL LICENSE BLOCK #############################



;;
;;  ENTITY AI SYSTEM
;;
.include "cpctelera.h.s"
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "man/entity_ai.h.s"
.include "sys/patrol.h.s"

.module entity_ai_system

;e_ai_st_noAI        = 0
;e_ai_st_stand_by    = 1
;e_ai_st_move_to     = 2
;e_ai_st_patrol      = 3
;e_ai_st_seek        = 4
;e_ai_st_projectile  = 5

_ia_states::
    .dw nullptr
    .dw entity_ai_sys_stand_by
    .dw entity_ai_sys_move_to
    .dw entity_ai_sys_patrol
    .dw entity_ai_sys_seek

;;
;; INICIALIZACION
;;
entity_ai_sys_init::
    call entity_man_getArray
    ld (_ent_array_ptr_seek), hl

;; solo para array de punteros
    call entity_ai_man_getArrayHL
    ld (_ent_array_ptr), hl
    ret

;;
;; ACTUALIZAR
;;
entity_ai_sys_update::

_ent_array_ptr = .+1
    ld hl, #0x0000

_loop:
;;  para recorrer un array de punteros
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl

    ld a, e
    or d
    ret z

    ld__ixl_e
    ld__ixh_d

    push hl

;; 6 diagrama de clases
;; 7 menu y game
;; 43 descompresor
;;

_bp::
    ld hl, #_ia_states
    ld a, e_ai_st(ix)
    add a
    add_hl_a
    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a
    ld (_ia_ptr), hl
_ia_ptr = .+1
    call #0x0000

    pop  hl

    jr _loop

;;
;; STAND BY
;;
entity_ai_sys_stand_by::
;;    ld e_ai_pre_st(ix), #e_ai_st_stand_by
;;    ld e_ai_st(ix), #e_ai_st_patrol

    ret

;;
;; MOVE TO
;;
entity_ai_sys_move_to::
    ld e_vx(ix), #0
    ld a, e_ai_aim_x(ix)
    sub e_x(ix)
    jr nc, _objx_greater_or_equal

_objx_lesser:
    ld e_vx(ix), #-1
    jr _endif_objx

_objx_greater_or_equal:
    jr z, _objx_target
    ld e_vx(ix), #1
    jr _endif_objx

_objx_target:
    ld e_vx(ix), #0

_endif_objx:

    ld e_vy(ix), #0

    ld a, e_ai_aim_y(ix)
    sub e_y(ix)
    jr nc, _objy_greater_or_equal

_objy_lesser:
    ld e_vy(ix), #-2
    jr _endif_objy

_objy_greater_or_equal:
    jr z, _objy_target
    ld e_vy(ix), #2
    jr _endif_objy

_objy_target:
    ld e_vy(ix), #0
    
    ld a, e_vx(ix)
    or a
    jr nz, _endif_objy

    ld a, e_ai_pre_st(ix)
    ld e_ai_st(ix), a
    ld e_ai_pre_st(ix), #e_ai_st_move_to

_endif_objy:

    ret

;;
;; PATROL
;;
entity_ai_sys_patrol::
    ld l, e_ai_patrol_step_l(ix)
    ld h, e_ai_patrol_step_h(ix)
    
    ld a, (hl)
    cp #patrol_invalid_move
    jr z, _reset_patrol

    ld e_ai_aim_x(ix), a
    inc hl
    ld a, (hl)
    ld e_ai_aim_y(ix), a

    inc hl
    ld e_ai_patrol_step_l(ix), l
    ld e_ai_patrol_step_h(ix), h

    ld e_ai_pre_st(ix), #e_ai_st_patrol
    ld e_ai_st(ix), #e_ai_st_move_to
    ret

_reset_patrol:
    inc hl
    ld a, (hl)
    inc hl
    ld h, (hl)

    ld e_ai_patrol_step_l(ix), a
    ld e_ai_patrol_step_h(ix), h
    ret


;;
;; SEEK
;;
entity_ai_sys_seek::
_ent_array_ptr_seek = .+2
    ld iy, #0x0000

    call entity_ai_sys_check_proximity
    ret c
    
    ld a, e_x(iy)
    ld e_ai_aim_x(ix), a
    ld a, e_y(iy)
    ld e_ai_aim_y(ix), a
    ld e_ai_pre_st(ix), #e_ai_st_seek
    ld e_ai_st(ix), #e_ai_st_move_to

    ret

entity_ai_sys_check_proximity::
;; IY player IX alien
;; si hay carry no esta cerca
    ld a, e_endx(iy)
    add #10
    sub e_x(ix)
    ret c
    
    ld a, e_endx(ix)
    add #10
    sub e_x(iy)
    ret c

    ld a, e_endy(iy)
    add #10
    sub e_y(ix)
    ret c
    
    ld a, e_endy(ix)
    add #10
    sub e_y(iy)
    ret