;;########################## BEGIN GPL LICENSE BLOCK #############################
;;    This file is part of Area51Raid.
;;
;;    Area51Raid is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Area51Raid is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Area51Raid.  If not, see <https://www.gnu.org/licenses/>.
;;
;;    Developed for the CPCRetrodev contest 2019 edition.
;;
;;    Developers                           Contact
;;    = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
;;    Aguirre Ivorra, Antonio              antoni.aguirre@gmail.com
;;    López Gutiérrez, Juan Carlos         juancarlos.logut@gmail.com
;;    Pérez Ponsoda, Jorge Amil            j.a.perez.ponsoda@gmail.com
;;
;;########################### END GPL LICENSE BLOCK #############################

;;
;;  ENTITY RENDER SYSTEM
;;

.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "man/entity.h.s"
.include "cmp/entity.h.s"
.include "assets/assets.h.s"
.include "man/menu.h.s"
.include "sys/render.h.s"
.include "man/render.h.s"

.module render_system

;; Pointers to the screen and back buffer
screen_start: .db  0xC0
buffer_start: .db  0x80

;; INICIALIZACION
;;  INPUT:
;;    -
;;  DESTROY:
;;    AF, BC, HL
render_sys_init::
    call entity_render_man_getArrayHL
    ld  (_ent_array_ptr), hl

    ld  hl, #init_render_entity
    ld  (_method_to_render), hl

    ld c, #0                        ;; [2]
    call cpct_setVideoMode_asm      ;; [x]
    ld hl, #_pal_main               ;; [3]
    ld de, #16                      ;; [3]
    call cpct_setPalette_asm        ;; [x]
    cpctm_setBorder_asm HW_WHITE    ;; [x]

    push af

    ld   l, #0xAE
    call cpct_setBlendMode_asm

    pop  af

    ret

;;
;; ACTUALIZAR
;;
;;  INPUT:
;;      -
;;  DESTROY:
;;      AF, BC, DE, HL, IX
init_render_sys_update::
    ld  hl, #init_render_entity
    ld  (_method_to_render), hl

render_sys_update::

_ent_array_ptr = .+1
    ld  hl, #0x0000

_loop:
    ld   e, (hl)
    inc hl
    ld   d, (hl)
    inc hl

    ld   a, e
    or   d
    jr   z, _end_loop

    ld__ixl_e
    ld__ixh_d

    push hl

_method_to_render = .+1
    call 0x0000

    pop hl

    jr _loop

_end_loop:
    ld  hl, #render_entity
    ld  (_method_to_render), hl

    ret

;;
;; Remove the last sprite in the back buffer and draw the sprite (blended) in a new
;; position inside the back buffer, updating the pointers for its deleted back
;;
;;  INPUT:
;;      IX: Pointer to the entity to draw
;;  DESTROY:
;;      AF, BC, DE, HL
render_entity:
    ;; Se borra la entidad del buffer secundario
    ld   e, e_lastVPB_l(ix)             ;; [6]
    ld   d, e_lastVPB_h(ix)             ;; [6]
    ld   l, e_pspr_l(ix)                ;; [6]
    ld   h, e_pspr_h(ix)                ;; [6]

    ld   b, e_w(ix)                     ;; [6]
    ld   c, e_h(ix)                     ;; [6]
    push bc                             ;; [4]

    call cpct_drawSpriteBlended_asm     ;; [x]

    ;; Se actualiza el puntero al buffer secundrio
    ld   e, e_lastVPS_l(ix)             ;; [6]
    ld   d, e_lastVPS_h(ix)             ;; [6]
    
    ld   e_lastVPB_l(ix), e             ;; [6]
    ld   e_lastVPB_h(ix), d             ;; [6]

    ;; Se pinta el sprite en el buffer secundario
    ld   a, (buffer_start)
    ld   d, a                           ;; [3]
    ld   e, #0
    ld   c, e_x(ix)                     ;; [6]
    ld   b, e_y(ix)                     ;; [6]
    call cpct_getScreenPtr_asm          ;; [x]

    ld  e_lastVPS_l(ix), l              ;; [6]
    ld  e_lastVPS_h(ix), h              ;; [6]_ent_array_update_ptr

    ex  de, hl                          ;; [1]
    ld   l, e_pspr_l(ix)                ;; [6]
    ld   h, e_pspr_h(ix)                ;; [6]
    pop bc                              ;; [3]

    call cpct_drawSpriteBlended_asm     ;; [x]

    ret

;;
;;  Draws sprite blended in the screen and in the back buffer  
;;
;;  INPUT:
;;      IX: Pointer to the entity to draw
;;  DESTROY:
;;      AF, BC, DE, HL
init_render_entity:
    ld   a, (screen_start)
    ld   d, a                           ;; [3]
    ld   e, #0
    ld   c, e_x(ix)                     ;; [6]
    ld   b, e_y(ix)                     ;; [6]
    call cpct_getScreenPtr_asm          ;; [x]

    ld  e_lastVPS_l(ix), l              ;; [6]
    ld  e_lastVPS_h(ix), h              ;; [6]_ent_array_update_ptr

    ex  de, hl                          ;; [1]
    ld   l, e_pspr_l(ix)                ;; [6]
    ld   h, e_pspr_h(ix)                ;; [6]
    ld   b, e_w(ix)                     ;; [6]
    ld   c, e_h(ix)                     ;; [6]
    
    call cpct_drawSpriteBlended_asm     ;; [x]

    ld   a, (buffer_start)
    ld   d, a                           ;; [3]
    ld   e, #0
    ld   c, e_x(ix)                     ;; [6]
    ld   b, e_y(ix)                     ;; [6]
    call cpct_getScreenPtr_asm          ;; [x]

    ld  e_lastVPB_l(ix), l              ;; [6]
    ld  e_lastVPB_h(ix), h              ;; [6]_ent_array_update_ptr

    ex  de, hl                          ;; [1]
    ld   l, e_pspr_l(ix)                ;; [6]
    ld   h, e_pspr_h(ix)                ;; [6]
    ld   b, e_w(ix)                     ;; [6]
    ld   c, e_h(ix)                     ;; [6]
    
    call cpct_drawSpriteBlended_asm     ;; [x]

    ret

set_background::
    ld  (_map_to_draw_0), hl
    ld  (_map_to_draw_1), hl
    
    ret

draw_background::
    ld  hl, #_palette
    ld  de, #16

;; hl: puntero a la memoria de video donde quiero pintar
;; de: puntero al tilemap (posicion inicial del tilemap)
    ld  hl, #0xC000
_map_to_draw_0 = .+1
    ld  de, #0x0000  ;; primer tile a pintar (si lo cambiamos, cambiamos el tile por donde empezamos.. )
 
;;  Destroyed Register values
;;  C-bindings	AF, BC, DE, HL
;;  ASM-bindings	AF, BC, DE, HL, IX, IY
    call cpct_etm_drawTilemap4x8_ag_asm

    ld  hl, #0x8000
_map_to_draw_1 = .+1
    ld  de, #0x0000  ;; primer tile a pintar (si lo cambiamos, cambiamos el tile por donde empezamos.. )

;;  Destroyed Register values
;;  C-bindings	AF, BC, DE, HL
;;  ASM-bindings	AF, BC, DE, HL, IX, IY
    call cpct_etm_drawTilemap4x8_ag_asm

    ret

;;  INPUT:
;;      -
;;  DESTROY:
;;      HL, BC, DE, A
clear_screen::
    ld   a, (screen_start)
    ld   h, a
    ld   l, #0
    ld  (hl), #0
    ex  de, hl
    inc  e
    ld   a, (screen_start)
    ld   h, a
    ld   l, #0
    ld  bc, #0x3FFE
    ldir

    ;? Borrado del buffer
    ld   a, (buffer_start)
    ld   h, a
    ld   l, #0
    ld  (hl), #1
    ex  de, hl
    inc  e
    ld   a, (buffer_start)
    ld   h, a
    ld   l, #0
    ld  bc, #0x3FFF
    ldir

    ret

;;  INPUT:
;;      -
;;  DESTROY:
;;      AF, BC, HL
switchBuffers::
    ld   a, (buffer_start)
    ld   b, a
    ld   a, (screen_start)
    ld  (buffer_start), a
    ld   a, b
    ld  (screen_start), a
    
    srl  b
    srl  b
    ld   l, b
    jp  cpct_setVideoMemoryPage_asm
    ;; Recursion de cola, no hace falta el ret

;;                                          ###############
;;                                          ##   MENUS   ##
;;                                          ###############

;; ########################
;; # RENDER SYS DRAW MENU #  // Se puede mirar para cambiar el modo de la pantalla para poder escribir textos más pequeños sin necesidad ed hacer un sprite
;; ########################

;;  Draw a new menu
;;  
;;  INPUT:
;;      IX = structure pointer for draw menu
;;
render_sys_draw_menu::
    ;; Cargo en hl los colores para pintar el string
    ld   l, (ix)
    inc ix
    ld   h, (ix)
    call cpct_setDrawCharM0_asm
    inc ix

next_string:
    ;; Compruebo si estoy en el final del string
    ld  a, (ix)
    cp  #end_str

    ret z

    ;; Se carga el puntero donde empieza la pantalla para clcular donde se va a pintar
    ld   a, (screen_start)
    ld   d, a
    ld   e, #0
    ld   c, (ix)
    inc ix
    ld   b, (ix)
    inc ix
    call cpct_getScreenPtr_asm

    ;; Se pinta el string
    ld  iy, #0x0000
    ld  e, (ix)
    inc ix
    ld  d, (ix)
    add iy, de
    push ix
    call cpct_drawStringM0_asm
    pop  ix
    inc ix

    jr next_string

;; ##########################
;; # RENDER SYS DELETE MENU #
;; ##########################

;;render_sys_draw_menu::
;;    ;; Cargo en hl los colores para pintar el string
;;    ld   l, (ix)
;;    inc ix
;;    ld   h, (ix)
;;    call cpct_setDrawCharM0_asm
;;    inc ix
;;
;;next_string:
;;    ;; Compruebo si estoy en el final del string
;;    ld  a, (ix)
;;    cp  #end_str
;;
;;    ret z
;;
;;    ;; Se carga el puntero donde empieza la pantalla para clcular donde se va a pintar
;;    ld   a, (screen_start)
;;    ld   d, a
;;    ld   e, #0
;;    ld   c, (ix)
;;    inc ix
;;    ld   b, (ix)
;;    inc ix
;;    call cpct_getScreenPtr_asm
;;
;;    ;; Se pinta el string
;;    ld  iy, #0x0000
;;    ld  e, (ix)
;;    inc ix
;;    ld  d, (ix)
;;    add iy, de
;;    push ix
;;    call cpct_drawStringM0_asm
;;    pop  ix
;;    inc ix
;;
;;    jr next_string
;;
;;

render_sys_remove_menu::
_prts_to_remove = .+2
    ld  ix, #0x0000

    
rem_next_str::
    ;; Si ha terminado se sale
    ld  a, (ix)
    cp  #end_str

    ret z

    ld   c, a

    ld   a, (screen_start)
    ld   d, a
    ld   e, #0
    inc ix
    ld   b, (ix)
    call cpct_getScreenPtr_asm
    inc ix

    ex  de, hl
    ld   a, (ix)
    inc ix
    ld   c, (ix)
    inc ix
    ld   b, (ix)
    call cpct_drawSolidBox_asm

    inc ix

    jr rem_next_str

;;;; ##########################
;;;; # RENDER SYS DELETE MENU #
;;;; ##########################
;;
;;;;
;;;;  Store structure pointer for deletion
;;;;  
;;;;  INPUT:
;;;;      HL = structure pointer for deletion
;;;;
set_array_to_remove::
    ld (_prts_to_remove), hl

    ret

;;;;
;;;;  Delete the current menu
;;;;
;;;; Destroyed Register values: 
;;;;      AF, BC, DE, HL, IX
;;;;
;;render_sys_remove_menu::
;;    _prts_to_remove = .+2
;;    ld  ix, #0x0000
;;
;;    
;;rem_next_str::
;;    ;; Si ha terminado se sale
;;    ld  a, (ix)
;;    cp  #end_str
;;
;;    ret z
;;
;;    ld   c, a
;;
;;    ld   a, (screen_start)
;;    ld   d, a
;;    ld   e, #0
;;    inc ix
;;    ld   b, (ix)
;;    call cpct_getScreenPtr_asm
;;    inc ix
;;
;;    ex  de, hl
;;    ld   a, (ix)
;;    inc ix
;;    ld   c, (ix)
;;    inc ix
;;    ld   b, (ix)
;;    call cpct_drawSolidBox_asm
;;
;;    inc ix
;;
;;    jr rem_next_str
;;
